import sys
import yaml
from os import path

with open(sys.argv[1]) as f:
    seed = yaml.load(f)

i = 0
snaps = seed["snaps"]
while i < len(snaps):
    entry = snaps[i]
    if entry["name"] == "pc":
        snaps[i] = {
            "name": "pc",
            "unasserted": True,
            "file": "pc_x1.snap",
        }
        break
    i += 1

for snap in sys.argv[2:]:
    # test-snapd-with-configure_123.snap -> test-snapd-configure
    snapname = path.basename(snap).split("_")[0]
    snaps.append(
        {"name": snapname, "channel": "edge", "file": snap,}
    )

with open(sys.argv[1], "w") as f:
    yaml.dump(seed, stream=f, indent=2, default_flow_style=False)



"""
Pure-python limited-use wrapper around the mount library function.
"""
from __future__ import print_function, absolute_import, unicode_literals

from argparse import ArgumentParser
from ctypes import CDLL, c_char_p, c_long, get_errno
from ctypes.util import find_library
from os import strerror
from sys import stderr, version_info

try:
    from typing import Text
except ImportError:
    pass

__all__ = ("mount",)


PY2 = version_info[0] == 2


class MountOpts(object):
    """MountOpts contain various flags for the mount system call."""

    Bind = 4096


def mount(source, target, fstype, flags=0, data=""):
    # type: (Text, Text, Text, int, Text) -> None
    """mount is a thin wrapper around the mount library function."""
    if PY2:
        c = b"c"
    else:
        c = "c"
    libc_name = find_library(c)
    if libc_name is None:
        raise Exception("cannot find the C library")
    libc = CDLL(libc_name, use_errno=True)
    retval = libc.mount(
        c_char_p(source.encode("UTF-8")),
        c_char_p(target.encode("UTF-8")),
        c_char_p(fstype.encode("UTF-8")),
        c_long(flags),
        None if data == "" else c_char_p(data.encode("UTF-8")),
    )
    if retval < 0:
        errno = get_errno()
        raise OSError(errno, strerror(errno))


def main():
    # type: () -> None
    parser = ArgumentParser()
    parser.add_argument("source", help="path of the device or bind source")
    parser.add_argument("target", help="path of the new mount point")
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument(
        "--bind",
        action="store_const",
        const="bind",
        help="create a new mount point of existing path",
    )
    group.add_argument("-t", "--type", help="filesystem type to mount")
    opts = parser.parse_args()
    if opts.bind is not None:
        mount(opts.source, opts.target, "", MountOpts.Bind)
    else:
        mount(opts.source, opts.target, opts.fstype)


if __name__ == "__main__":
    try:
        main()
    except OSError as err:
        print(err, file=stderr)
        raise SystemExit(1)



#!/usr/bin/env python3

import argparse
import logging
import yaml
import sys


def parse_arguments():
    parser = argparse.ArgumentParser(description="ensure pc gadget has ubuntu-save")
    parser.add_argument(
        "gadgetyaml", type=argparse.FileType("r"), help="path to gadget.yaml input file"
    )
    return parser.parse_args()


def main(opts):
    gadget_yaml = yaml.safe_load(opts.gadgetyaml)

    structs = gadget_yaml["volumes"]["pc"]["structure"]
    save_idx = -1
    for idx, s in enumerate(structs):
        role = s.get("role", "")
        if role == "system-save":
            logging.info("system-save structure already present")
            # already has ubuntu-save
            return
        if role == "system-data":
            # ubuntu-save precedes ubuntu-data
            save_idx = idx
            break
    if save_idx == -1:
        raise RuntimeError("cannot find a suitable place to insert ubuntu-save")

    ubuntu_save = {
        "name": "ubuntu-save",
        "role": "system-save",
        # TODO:UC20: update when pc-amd64-gadget changes
        "size": "16M",
        "filesystem": "ext4",
        "type": "83,0FC63DAF-8483-4772-8E79-3D69D8477DE4",
    }
    structs.insert(save_idx, ubuntu_save)
    yaml.dump(gadget_yaml, stream=sys.stdout, indent=2, default_flow_style=False)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    opts = parse_arguments()
    main(opts)



#!/usr/bin/python3

# Tiny HTTP Proxy. Based on the work of SUZUKI Hisao.
#
# Ported to py3 and modified to remove the bits we don't need
# and modernized.

import os
import http.server
import select
import socket
import socketserver
import sys
import urllib.parse


class ProxyHandler(http.server.BaseHTTPRequestHandler):
    server_version = "testsproxy/1.0"

    def log_request(self, m=""):
        super().log_request(m)
        sys.stdout.flush()
        sys.stderr.flush()

    def handle(self):
        (ip, port) = self.client_address
        super().handle()

    def _connect_to(self, netloc, soc):
        i = netloc.find(":")
        if i >= 0:
            host_port = netloc[:i], int(netloc[i + 1 :])
        else:
            host_port = netloc, 80
        try:
            soc.connect(host_port)
        except socket.error as arg:
            try:
                msg = arg[1]
            except:
                msg = arg
            self.send_error(404, msg)
            return False
        return True

    def do_CONNECT(self):
        soc = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            if self._connect_to(self.path, soc):
                self.log_request(200)
                s = self.protocol_version + " 200 Connection established\r\n"
                self.wfile.write(s.encode())
                s = "Proxy-agent: {}\r\n".format(self.version_string())
                self.wfile.write(s.encode())
                self.wfile.write("\r\n".encode())
                self._read_write(soc, 300)
        finally:
            soc.close()
            self.connection.close()

    def do_GET(self):
        (scm, netloc, path, params, query, fragment) = urllib.parse.urlparse(
            self.path, "http"
        )
        if scm != "http" or fragment or not netloc:
            s = "bad url {}".format(self.path)
            self.send_error(400, s.encode())
            return
        soc = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            if self._connect_to(netloc, soc):
                self.log_request()
                s = "{} {} {}\r\n".format(
                    self.command,
                    urllib.parse.urlunparse(("", "", path, params, query, "")),
                    self.request_version,
                )
                soc.send(s.encode())
                self.headers["Connection"] = "close"
                del self.headers["Proxy-Connection"]
                for key, val in self.headers.items():
                    s = "{}: {}\r\n".format(key, val)
                    soc.send(s.encode())
                soc.send("\r\n".encode())
                self._read_write(soc)
        finally:
            soc.close()
            self.connection.close()

    def _read_write(self, soc, max_idling=20):
        iw = [self.connection, soc]
        ow = []
        count = 0
        while True:
            count += 1
            (ins, _, exs) = select.select(iw, ow, iw, 3)
            if exs:
                break
            if ins:
                for i in ins:
                    if i is soc:
                        out = self.connection
                    else:
                        out = soc
                    data = i.recv(8192)
                    if data:
                        out.send(data)
                        count = 0
            if count == max_idling:
                break

    do_HEAD = do_GET
    do_POST = do_GET
    do_PUT = do_GET
    do_DELETE = do_GET


def maybe_sd_notify(s: str) -> None:
    addr = os.getenv("NOTIFY_SOCKET")
    if not addr:
        return
    soc = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
    soc.connect(addr)
    soc.sendall(s.encode())


class ThreadingHTTPServer(socketserver.ThreadingMixIn, http.server.HTTPServer):
    def __init__(self, *args):
        super().__init__(*args)
        maybe_sd_notify("READY=1")


if __name__ == "__main__":
    port = 3128
    print("starting tinyproxy on port {}".format(port))
    http.server.test(ProxyHandler, ThreadingHTTPServer, port=port)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from __future__ import print_function

# XXX copied from https://github.com/kyrofa/cla-check
# (and then heavily modified)
import os
import re
import sys
import argparse
from subprocess import check_call, check_output


try:
    from launchpadlib.launchpad import Launchpad
except ImportError:
    sys.exit(
        "Install launchpadlib: sudo apt install python-launchpadlib python3-launchpadlib"
    )

shortlog_email_rx = re.compile(r"^\s*\d+\s+.*<(\S+)>$", re.M)


is_travis = os.getenv("TRAVIS", "") == "true"
is_github_actions = os.getenv("GITHUB_ACTIONS", "") == "true"


def get_emails_for_range(r):
    output = check_output(["git", "shortlog", "-se", r]).decode("utf-8")
    return set(m.group(1) for m in shortlog_email_rx.finditer(output))


if sys.stdout.isatty() or is_travis or is_github_actions:
    green = "\033[32;1m"
    red = "\033[31;1m"
    yellow = "\033[33;1m"
    reset = "\033[0m"
    clear = "\033[0K"
else:
    green = ""
    red = ""
    yellow = ""
    reset = ""
    clear = ""

if is_travis:
    fold_start = "travis_fold:start:{{tag}}\r{}{}{{message}}{}".format(
        clear, yellow, reset
    )
    fold_end = "travis_fold:end:{{tag}}\r{}".format(clear)
elif is_github_actions:
    fold_start = "::group::{message}"
    fold_end = "::endgroup::"
else:
    fold_start = "{}{{message}}{}".format(yellow, reset)
    fold_end = ""


def static_email_check(email, master_emails, width):
    if email in master_emails:
        print("{}✓{} {:<{}} already on master".format(green, reset, email, width))
        return True
    if email.endswith("@canonical.com"):
        print("{}✓{} {:<{}} @canonical.com account".format(green, reset, email, width))
        return True
    if email.endswith("@mozilla.com"):
        print(
            "{}✓{} {:<{}} @mozilla.com account (mozilla corp has signed the corp CLA)".format(
                green, reset, email, width
            )
        )
        return True
    if email.endswith("@users.noreply.github.com"):
        print(
            "{}‽{} {:<{}} privacy-enabled github web edit email address".format(
                yellow, reset, email, width
            )
        )
        return False
    return False


def lp_email_check(email, lp, cla_folks, width):
    contributor = lp.people.getByEmail(email=email)
    if not contributor:
        print("{}🛇{} {:<{}} has no Launchpad account".format(red, reset, email, width))
        return False

    if contributor in cla_folks:
        print(
            "{}✓{} {:<{}} ({}) has signed the CLA".format(
                green, reset, email, width, contributor
            )
        )
        return True
    else:
        print(
            "{}🛇{} {:<{}} ({}) has NOT signed the CLA".format(
                red, reset, email, width, contributor
            )
        )
        return False


def print_checkout_info(travis_commit_range):
    # This is just to have information in case things go wrong
    print(fold_start.format(tag="checkout_info", message="Debug information"))
    print("Commit range:", travis_commit_range)
    print("Remotes:")
    sys.stdout.flush()
    check_call(["git", "remote", "-v"])
    print("Branches:")
    sys.stdout.flush()
    check_call(["git", "branch", "-v"])
    sys.stdout.flush()
    print(fold_end.format(tag="checkout_info"))


def main():
    parser = argparse.ArgumentParser(description="")
    parser.add_argument(
        "commit_range", help="Commit range in format <upstream-head>..<fork-head>"
    )
    opts = parser.parse_args()
    master, _ = opts.commit_range.split("..")
    print_checkout_info(opts.commit_range)
    emails = get_emails_for_range(opts.commit_range)
    if len(emails) == 0:
        sys.exit("No emails found in in the given commit range.")

    master_emails = get_emails_for_range(master)

    width = max(map(len, emails))
    lp = None
    failed = False
    print("Need to check {} emails:".format(len(emails)))
    for email in emails:
        if static_email_check(email, master_emails, width):
            continue
        # in the normal case this isn't reached
        if lp is None:
            print("Logging into Launchpad...")
            lp = Launchpad.login_anonymously("check CLA", "production")
            cla_folks = lp.people["contributor-agreement-canonical"].participants
        if not lp_email_check(email, lp, cla_folks, width):
            failed = True

    if failed:
        sys.exit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/python3
#
# see http://daringfireball.net/projects/markdown/syntax
# for the "canonical" reference
#
# We support django-markdown which uses python-markdown, see:
# http://pythonhosted.org/Markdown/

import sys
import codecs


def lint_li(fname, text):
    """Ensure that the list-items are multiplies of 4"""
    is_clean = True
    for i, line in enumerate(text.splitlines()):
        if line.lstrip().startswith("*") and line.index("*") % 4 != 0:
            print("%s: line %i list has non-4 spaces indent" % (fname, i))
            is_clean = False
    return is_clean


def lint(md_files):
    """lint all md files"""
    all_clean = True
    for md in md_files:
        with codecs.open(md, "r", "utf-8") as f:
            buf = f.read()
            for fname, func in globals().items():
                if fname.startswith("lint_"):
                    all_clean &= func(md, buf)
    return all_clean


if __name__ == "__main__":
    if not lint(sys.argv):
        sys.exit(1)



#!/usr/bin/python3

import os
import sys

import dbus.service
import dbus.mainloop.glib
from gi.repository import GLib

BUS_NAME = "org.freedesktop.impl.portal.spread"
OBJECT_PATH = "/org/freedesktop/portal/desktop"

APP_CHOOSER_IFACE = "org.freedesktop.impl.portal.AppChooser"
FILE_CHOOSER_IFACE = "org.freedesktop.impl.portal.FileChooser"
SCREENSHOT_IFACE = "org.freedesktop.impl.portal.Screenshot"


class PortalImpl(dbus.service.Object):
    def __init__(self, connection, object_path, config):
        super(PortalImpl, self).__init__(connection, object_path)
        self._config = config

    @dbus.service.method(
        dbus_interface=FILE_CHOOSER_IFACE,
        in_signature="osssa{sv}",
        out_signature="ua{sv}",
    )
    def OpenFile(self, handle, app_id, parent_window, title, options):
        return (
            0,
            dict(
                uris=dbus.Array(["file:///tmp/file-to-read.txt"], signature="s"),
                writable=False,
            ),
        )

    @dbus.service.method(
        dbus_interface=FILE_CHOOSER_IFACE,
        in_signature="osssa{sv}",
        out_signature="ua{sv}",
    )
    def SaveFile(self, handle, app_id, parent_window, title, options):
        return (
            0,
            dict(
                uris=dbus.Array(["file:///tmp/file-to-write.txt"], signature="s"),
                writable=True,
            ),
        )

    @dbus.service.method(
        dbus_interface=SCREENSHOT_IFACE, in_signature="ossa{sv}", out_signature="ua{sv}"
    )
    def Screenshot(self, handle, app_id, parent_window, options):
        return (0, dict(uri="file:///tmp/screenshot.txt"))

    @dbus.service.method(
        dbus_interface=APP_CHOOSER_IFACE,
        in_signature="ossasa{sv}",
        out_signature="ua{sv}",
    )
    def ChooseApplication(self, handle, app_id, parent_window, choices, options):
        if options["content_type"] == "text/plain":
            return (0, dict(choice="test-editor"))
        else:
            return (1, {})


def main(argv):
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    main_loop = GLib.MainLoop()

    bus = dbus.SessionBus()
    # Make sure we quit when the bus shuts down
    bus.add_signal_receiver(
        main_loop.quit,
        signal_name="Disconnected",
        path="/org/freedesktop/DBus/Local",
        dbus_interface="org.freedesktop.DBus.Local",
    )

    portal = PortalImpl(bus, OBJECT_PATH, None)

    # Allow other services to assume our bus name
    bus_name = dbus.service.BusName(
        BUS_NAME, bus, allow_replacement=True, replace_existing=True, do_not_queue=True
    )

    main_loop.run()


if __name__ == "__main__":
    sys.exit(main(sys.argv))



#!/usr/bin/env python3

import configparser
import hashlib
import json
import logging
import os
import psutil
import time

PROCATTRS = "proc.attrs"
PROCS = "proc.names"
RATE = "iter.rate"
INTERVAL = "iter.interval"
RATE_CONFIG = "iter.rate.config"

SNAP_DIR = os.getenv("SNAP", ".")
COMMON_DIR = os.getenv("SNAP_COMMON", ".")
LOG_PATH = os.path.join(COMMON_DIR, "profiler.log")
DEFAULTS_PATH = os.path.join(SNAP_DIR, "etc", "config.ini")
CONFIG_PATH = os.path.join(COMMON_DIR, "profiler.conf")
CONFIG_PATH_FLAG = os.path.join(COMMON_DIR, "reconfigured")


def prepare_config(config):
    new_config = dict(config)
    new_config[PROCATTRS] = [proc.strip() for proc in config.get(PROCATTRS).split(",")]
    new_config[PROCS] = [proc.strip() for proc in config.get(PROCS).split(",")]
    new_config[INTERVAL] = float(config.get(INTERVAL, 1))
    for key, val in config.items():
        if RATE in key:
            new_config[key] = int(config.get(key, 1))

    logging.info("Using config: {}".format(new_config))
    return new_config


def read_config(config_path):
    config = configparser.ConfigParser()
    if os.path.isfile(config_path):
        config.read(config_path)
        return prepare_config(config["DEFAULT"])
    else:
        logging.error("Config file {} not found".format(config_path))
        exit(1)


def get_config():
    config_path = DEFAULTS_PATH
    if os.path.isfile(CONFIG_PATH):
        config_path = CONFIG_PATH

    return read_config(config_path)


def check_config():
    if os.path.isfile(CONFIG_PATH) and not os.path.isfile(CONFIG_PATH_FLAG):
        new_config = read_config(CONFIG_PATH)
        open(CONFIG_PATH_FLAG, "a").close()
        return new_config
    else:
        return None


def main():
    logging.basicConfig(
        filename=LOG_PATH,
        filemode="w",
        level=logging.INFO,
        format="%(asctime)s - %(message)s",
    )
    config = get_config()

    count = 0
    while True:
        if count % config.get(RATE_CONFIG, 1) == 0:
            new_config = check_config()
            if new_config:
                config = new_config

        procs = [
            proc
            for proc in psutil.process_iter()
            if proc.name() in config.get(PROCS, [])
        ]
        for proc in procs:
            if count % config.get("{}.{}".format(RATE, proc.name()), 1) == 0:
                logging.info(json.dumps(proc.as_dict(attrs=config.get(PROCATTRS))))

        time.sleep(config.get(INTERVAL))
        count = count + 1


if __name__ == "__main__":
    main()



#!/usr/bin/python3

import os
import sys
import urllib.request

from http.server import HTTPServer, SimpleHTTPRequestHandler


class XkcdRequestHandler(SimpleHTTPRequestHandler):

    XKCD_URL = "http://xkcd.com/"
    XKCD_IMG_URL = "http://imgs.xkcd.com/"

    def _mini_proxy(self, url):
        fp = urllib.request.urlopen(url)
        body = fp.read()
        info = fp.info()
        self.send_response(200, "ok")
        for k, v in info.items():
            self.send_header(k, v)
            self.end_headers()
            self.wfile.write(body)

    def do_GET(self):
        if self.path.startswith("/xkcd/"):
            url = self.XKCD_URL + self.path[len("/xkcd/") :]
            return self._mini_proxy(url)
        elif self.path.startswith("/img/xkcd/"):
            url = self.XKCD_IMG_URL + self.path[len("/img/xkcd/") :]
            return self._mini_proxy(url)
        else:
            return super(XkcdRequestHandler, self).do_GET()


if __name__ == "__main__":
    # we start in the snappy base directory, ensure we are in "www"
    os.chdir(os.path.dirname(__file__) + "/../www")

    if len(sys.argv) > 1:
        port = int(sys.argv[1])
    else:
        port = 80

    httpd = HTTPServer(("", port), XkcdRequestHandler)
    httpd.serve_forever()



#!/usr/bin/env python3

from gi.repository import GLib
import dbus
import dbus.service

from dbus.mainloop.glib import DBusGMainLoop

DBusGMainLoop(set_as_default=True)


class DBusProvider(dbus.service.Object):
    def __init__(self):
        bus = dbus.SessionBus()
        bus_name = dbus.service.BusName(
            "com.canonical.Autopilot.Introspection", bus=bus
        )
        dbus.service.Object.__init__(
            self, bus_name, "/com/canonical/Autopilot/Introspection"
        )

    @dbus.service.method(
        dbus_interface="com.canonical.Autopilot.Introspection", out_signature="s"
    )
    def GetVersion(self):
        return "my-ap-version"

    @dbus.service.method(
        dbus_interface="com.canonical.Autopilot.Introspection", out_signature="s"
    )
    def GetState(self):
        return "my-ap-state"


if __name__ == "__main__":
    DBusProvider()
    loop = GLib.MainLoop()
    loop.run()



#!/usr/bin/python3

import argparse
import re
import urllib.request

from html.parser import HTMLParser


class InvalidPRTitle(Exception):
    def __init__(self, invalid_title):
        self.invalid_title = invalid_title


class GithubTitleParser(HTMLParser):
    def __init__(self):
        HTMLParser.__init__(self)
        self._cur_tag = ""
        self.title = ""

    def handle_starttag(self, tag, attributes):
        self._cur_tag = tag

    def handle_endtag(self, tag):
        self._cur_tag = ""

    def handle_data(self, data):
        if self._cur_tag == "title":
            self.title = data


def check_pr_title(pr_number: int):
    # ideally we would use the github API - however we can't because:
    # a) its rate limiting and travis IPs hit the API a lot so we regularly
    #    get errors
    # b) using a API token is tricky because travis will not allow the secure
    #    vars for forks
    # so instead we just scrape the html title which is unlikely to change
    # radically
    parser = GithubTitleParser()
    with urllib.request.urlopen(
        "https://github.com/snapcore/snapd/pull/{}".format(pr_number)
    ) as f:
        parser.feed(f.read().decode("utf-8"))
    # the title has the format:
    #  "Added api endpoint for downloading snaps by glower · Pull Request #6958 · snapcore/snapd · GitHub"
    # so we rsplit() once to get the title (rsplit to not get confused by
    # possible "by" words in the real title)
    title = parser.title.rsplit(" by ", maxsplit=1)[0]
    print(title)
    # cover most common cases:
    # package: foo
    # package, otherpackage/subpackage: this is a title
    # tests/regression/lp-12341234: foo
    # [RFC] foo: bar
    if not re.match(r"[a-zA-Z0-9_\-\*/,. \[\]{}]+: .*", title):
        raise InvalidPRTitle(title)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "pr_number", metavar="PR number", help="the github PR number to check"
    )
    args = parser.parse_args()
    try:
        check_pr_title(args.pr_number)
    except InvalidPRTitle as e:
        print('Invalid PR title: "{}"\n'.format(e.invalid_title))
        print("Please provide a title in the following format:")
        print("module: short description")
        print("E.g.:")
        print("daemon: fix frobinator bug")
        raise SystemExit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/python3

import contextlib
import os
import fcntl
import re
import struct
import sys
from typing import Iterable


def if_open(dev: str) -> int:
    TUNSETIFF = 0x400454CA
    TUNSETOWNER = TUNSETIFF + 2
    IFF_TUN = 0x0001
    IFF_TAP = 0x0002
    IFF_NO_PI = 0x1000

    try:
        fd = os.open("/dev/net/tun", os.O_RDWR)
    except PermissionError as e:
        raise SystemExit(e)

    if_flags = None
    if dev.startswith("tun"):
        if_flags = IFF_TUN | IFF_NO_PI
    elif dev.startswith("tap"):
        if_flags = IFF_TAP | IFF_NO_PI

    fcntl.ioctl(fd, TUNSETIFF, struct.pack("16sH", str.encode(dev), if_flags))

    # for setting to owner
    # fcntl.ioctl(fd, TUNSETOWNER, 1000)

    return fd


def device_exists(dev: str) -> bool:
    return os.path.exists("/sys/devices/virtual/net/%s" % dev)


def valid_device_name(dev: str) -> None:
    if not re.search(r"^t(ap|un)[0-9]+$", dev):
        raise ValueError("device should be of form tun0-tun255 or tap0-tap255")

    if_num = int(dev[3:])
    if if_num > 255:
        raise ValueError("device should be of form tun0-tun255 or tap0-tap255")


@contextlib.contextmanager
def closing_fd(fd: int) -> Iterable[int]:
    try:
        yield fd
    finally:
        os.close(fd)


if __name__ == "__main__":
    if len(sys.argv) != 2:
        raise SystemExit("need to specify a tun/tap device (eg tun1 or tap2)")

    d = sys.argv[1]
    valid_device_name(d)

    if device_exists(d):
        raise SystemExit("ERROR: device '%s' already exists" % d)
        sys.exit(1)

    found = False
    with closing_fd(if_open(d)) as fd:
        if device_exists(d):
            found = True
            print("PASS")
        else:
            print("FAIL")

    if not found:
        sys.exit(1)



#!/usr/bin/python3

import argparse
import urllib.request
import logging

from html.parser import HTMLParser

LABELS = {
    # PR label indicating that spread job should be skipped
    'LABEL_SKIP_SPREAD_JOB': "Skip spread",
    # PR label indicating that nested tests need to be executed
    'LABEL_RUN_SPREAD_NESTED': "Run nested"
    }


class GithubLabelsParser(HTMLParser):
    def __init__(self):
        super().__init__()
        self.in_labels = False
        self.deep = 0
        self.labels = []

    def handle_starttag(self, tag, attributes):
        logging.debug(attributes)

        if not self.in_labels:
            attr_class = [attr[1] for attr in attributes if attr[0] == "class"]
            if len(attr_class) == 0:
                return
            # labels are in separate div defined like:
            # <div class=".. labels .." .. >
            elems = attr_class[0].split(" ")
            if "labels" in elems:
                self.in_labels = True
                self.deep = 1
                logging.debug("labels start")
        else:
            # nesting counter
            self.deep += 1

            # inside labels
            # label entry has
            # <a class=".." data-name="<label name>" />
            attr_data_name = [attr[1] for attr in attributes if attr[0] == "data-name"]
            if len(attr_data_name) == 0:
                return
            data_name = attr_data_name[0]
            logging.debug("found label: %s", data_name)
            self.labels.append(data_name)

    def handle_endtag(self, tag):
        if self.in_labels:
            self.deep -= 1
            if self.deep < 1:
                logging.debug("labels end")
                self.in_labels = False

    def handle_data(self, data):
        if self.in_labels:
            logging.debug("data: %s", data)


def grab_pr_labels(pr_number: int):
    # ideally we would use the github API - however we can't because:
    # a) its rate limiting and travis IPs hit the API a lot so we regularly
    #    get errors
    # b) using a API token is tricky because travis will not allow the secure
    #    vars for forks
    # so instead we just scrape the html title which is unlikely to change
    # radically
    parser = GithubLabelsParser()
    with urllib.request.urlopen(
        "https://github.com/snapcore/snapd/pull/{}".format(pr_number)
    ) as f:
        parser.feed(f.read().decode("utf-8"))
    return parser.labels


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "pr_number", 
        metavar="PR number", 
        help="the github PR number to check"
    )
    parser.add_argument(
        "label", 
        metavar="Label name", 
        choices=LABELS.keys(), 
        help="the github PR label to check"
    )
    parser.add_argument(
        "-d", "--debug", help="enable debug logging", action="store_true"
    )
    args = parser.parse_args()

    lvl = logging.INFO
    if args.debug:
        lvl = logging.DEBUG
    logging.basicConfig(level=lvl)

    pr_labels = grab_pr_labels(args.pr_number)
    print("pr labels:", pr_labels)

    if LABELS.get(args.label) not in pr_labels:
        raise SystemExit(1)

    print("Label: '{}' found in PR: '{}'".format(
        LABELS.get(args.label), args.pr_number))


if __name__ == "__main__":
    main()



#!/usr/bin/python3

import os
import sys

import dbus.service
import dbus.mainloop.glib
from gi.repository import GLib

BUS_NAME = "org.freedesktop.portal.Documents"
OBJECT_PATH = "/org/freedesktop/portal/documents"
DOC_IFACE = "org.freedesktop.portal.Documents"


class DocPortal(dbus.service.Object):
    def __init__(self, connection, object_path, logfile, errorfile):
        super(DocPortal, self).__init__(connection, object_path)
        self._logfile = logfile
        self._errorfile = errorfile

    @property
    def _report_error(self):
        with open(self._errorfile, "r") as fp:
            return "error" in fp.read()

    @dbus.service.method(dbus_interface=DOC_IFACE, in_signature="", out_signature="ay")
    def GetMountPoint(self):
        with open(self._logfile, "a") as fp:
            fp.write("GetMountPoint called\n")
        if self._report_error:
            raise dbus.DBusException("failure", name=DOC_IFACE + ".Error.Failed")
        return "/run/user/{}/doc\x00".format(os.getuid()).encode("ASCII")


def main(argv):
    logfile, errorfile = argv[1:]
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    main_loop = GLib.MainLoop()

    bus = dbus.SessionBus()
    # Make sure we quit when the bus shuts down
    bus.add_signal_receiver(
        main_loop.quit,
        signal_name="Disconnected",
        path="/org/freedesktop/DBus/Local",
        dbus_interface="org.freedesktop.DBus.Local",
    )

    portal = DocPortal(bus, OBJECT_PATH, logfile, errorfile)

    # Allow other services to assume our bus name
    bus_name = dbus.service.BusName(
        BUS_NAME, bus, allow_replacement=True, replace_existing=True, do_not_queue=True
    )

    main_loop.run()


if __name__ == "__main__":
    sys.exit(main(sys.argv))
